﻿#pragma once

#include <string>
#include "..\Common\DeviceResources.h"
#include "..\Common\StepTimer.h"
#include "RenderingOption.h"

namespace MeshTest
{
    class MeshTestRenderer
    {
    public:
        MeshTestRenderer(const std::shared_ptr<DX::DeviceResources>& deviceResources);
        void CreateDeviceDependentResources();
        void ReleaseDeviceDependentResources();
        void CreateWindowSizeDependentResources();
        void Update(DX::StepTimer const& timer);
        void Render();

        void SetRenderingOption(RenderingOption renderingOption);

    private:
        // Cached pointer to device resources.
        std::shared_ptr<DX::DeviceResources> m_deviceResources;

        // Resources related to text rendering.
        std::wstring                                    m_text;
        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_blueBrush;
        Microsoft::WRL::ComPtr<ID2D1DrawingStateBlock>  m_stateBlock;
        Microsoft::WRL::ComPtr<IDWriteTextFormat>		m_textFormat;

        // Geometry rendering
        RenderingOption                                   m_renderingOption = RenderingOption::None;
        float                                             m_rotationAngle;
        Microsoft::WRL::ComPtr<ID2D1PathGeometry>         m_starGeometry;
        Microsoft::WRL::ComPtr<ID2D1PathGeometry>         m_outlinedStarGeometry;
        Microsoft::WRL::ComPtr<ID2D1PathGeometry>         m_flattenedStarGeometry;
        Microsoft::WRL::ComPtr<ID2D1Mesh>                 m_starMesh;
        Microsoft::WRL::ComPtr<ID2D1GeometryRealization>  m_realizedStarGeometry;
    };
}