﻿#pragma once

#include "Common\StepTimer.h"
#include "Common\DeviceResources.h"
#include "Content\MeshTestRenderer.h"
#include "Content\RenderingOption.h"

// Renders Direct2D and 3D content on the screen.
namespace MeshTest
{
    class MeshTestMain : public DX::IDeviceNotify
    {
    public:
        MeshTestMain(const std::shared_ptr<DX::DeviceResources>& deviceResources);
        ~MeshTestMain();
        void CreateWindowSizeDependentResources();
        void StartTracking() {  }
        void TrackingUpdate(float positionX) { m_pointerLocationX = positionX; }
        void StopTracking() {  }
        bool IsTracking() { return false; }
        void StartRenderLoop();
        void StopRenderLoop();
        Concurrency::critical_section& GetCriticalSection() { return m_criticalSection; }

        // IDeviceNotify
        virtual void OnDeviceLost();
        virtual void OnDeviceRestored();

        void SetRenderingOption(RenderingOption renderingOption)
        {
            m_meshTestRenderer->SetRenderingOption(renderingOption);
        }

    private:
        void ProcessInput();
        void Update();
        bool Render();

        // Cached pointer to device resources.
        std::shared_ptr<DX::DeviceResources> m_deviceResources;

        // TODO: Replace with your own content renderers.
        std::unique_ptr<MeshTestRenderer> m_meshTestRenderer;

        Windows::Foundation::IAsyncAction^ m_renderLoopWorker;
        Concurrency::critical_section m_criticalSection;

        // Rendering loop timer.
        DX::StepTimer m_timer;

        // Track current input pointer position.
        float m_pointerLocationX;
    };
}