#include "pch.h"
#include "InterrogableTessellationSink.h"

using namespace D2D1;
using namespace DirectX;

// IUnknown methods
ULONG STDMETHODCALLTYPE InterrogableTessellationSink::AddRef()
{
    m_refCount++;
    return m_refCount;
}

ULONG STDMETHODCALLTYPE InterrogableTessellationSink::Release()
{
    m_refCount--;
    LONG newCount = m_refCount;

    if (m_refCount == 0)
        delete this;

    return newCount;
}

HRESULT STDMETHODCALLTYPE InterrogableTessellationSink::QueryInterface(_In_ REFIID riid, _Outptr_ void** ppOutput)
{
    *ppOutput = nullptr;
    HRESULT hr = S_OK;

    if (riid == __uuidof(ID2D1TessellationSink))
    {
        *ppOutput = static_cast<ID2D1TessellationSink*>(this);
        AddRef();
    }
    else if (riid == __uuidof(IUnknown))
    {
        *ppOutput = this;
        AddRef();
    }
    else
    {
        hr = E_NOINTERFACE;
    }
    return hr;
}

// Constructor
InterrogableTessellationSink::InterrogableTessellationSink() :
    m_refCount(0)
{
}

// ID2D1TessellationSink methods
void InterrogableTessellationSink::AddTriangles(_In_ const D2D1_TRIANGLE *triangles, 
                                                UINT trianglesCount)
{
    for (UINT i = 0; i < trianglesCount; i++)
    {
        m_triangles.push_back(triangles[i]);
    }
}

HRESULT InterrogableTessellationSink::Close()
{
    // Assume that the class accessing the tessellation sink knows what it's doing
    return S_OK;
}

// Method for this implementation
std::vector<D2D1_TRIANGLE> InterrogableTessellationSink::GetTriangles()
{
    return m_triangles;
}

