﻿#pragma once

#include <string>
#include "..\Common\DeviceResources.h"
#include "..\Common\StepTimer.h"

namespace OutThereAndBackAgain
{
    class OutThereAndBackAgainRenderer
    {
    public:
        OutThereAndBackAgainRenderer(const std::shared_ptr<DX::DeviceResources>& deviceResources);
        void CreateDeviceDependentResources();
        void ReleaseDeviceDependentResources();
        void CreateWindowSizeDependentResources();
        void SetRenderMode(bool useSingleFillMesh);
        void Update(DX::StepTimer const& timer);
        void Render();

    private:
        // Cached pointer to device resources.
        std::shared_ptr<DX::DeviceResources> m_deviceResources;

        // Resources for triangle rendering
        struct TriangleInfo
        {
            D2D1_TRIANGLE                               triangle;
            Microsoft::WRL::ComPtr<ID2D1Mesh>           triangleMesh;
            D2D1_POINT_2F                               triangleCenter;
            Microsoft::WRL::ComPtr<ID2D1PathGeometry>   travelGeometry;
            float                                       travelLength;
            float                                       initialAngle;
            D2D1_MATRIX_3X2_F                           matrix;
        };

        bool m_useSingleFillMesh;
        std::vector<TriangleInfo>                       m_triangleInfos;
        std::vector<D2D1_TRIANGLE>                      m_triangles;
        D2D1_RECT_F                                     m_geometryBounds;
        Microsoft::WRL::ComPtr<ID2D1LinearGradientBrush> m_gradientBrush;
        Microsoft::WRL::ComPtr<ID2D1Mesh>               m_textMesh;

        // Resources for FPS text rendering.
        std::wstring                                    m_text;
        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_blueBrush;
        Microsoft::WRL::ComPtr<ID2D1DrawingStateBlock>  m_stateBlock;
        Microsoft::WRL::ComPtr<IDWriteTextFormat>		m_textFormat;
    };
}