﻿#pragma once

#include <string>
#include "..\Common\DeviceResources.h"
#include "..\Common\StepTimer.h"

namespace SparklingText
{
    class SparklingTextRenderer
    {
    public:
        SparklingTextRenderer(const std::shared_ptr<DX::DeviceResources>& deviceResources);
        void CreateDeviceDependentResources();
        void ReleaseDeviceDependentResources();
        void CreateWindowSizeDependentResources();
        void SetRenderMode(bool useMeshesNotGeometries);
        void Tessellate();
        void Update(DX::StepTimer const& timer);
        void Render();

    private:
        // Cached pointer to device resources.
        std::shared_ptr<DX::DeviceResources> m_deviceResources;

        // Resources for triangle rendering
        bool                                            m_useMeshesNotGeometries;
        std::vector<Microsoft::WRL::ComPtr<ID2D1PathGeometry>>  m_triangleGeometries;
        std::vector<Microsoft::WRL::ComPtr<ID2D1Mesh>>  m_triangleMeshes;
        D2D1_RECT_F                                     m_geometryBounds;
        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_solidBrush;

        // Resources for FPS text rendering.
        std::wstring                                    m_text;
        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_whiteBrush;
        Microsoft::WRL::ComPtr<ID2D1DrawingStateBlock>  m_stateBlock;
        Microsoft::WRL::ComPtr<IDWriteTextFormat>		m_textFormat;
    };
}