﻿#pragma once

#include <string>
#include "..\Common\DeviceResources.h"
#include "..\Common\StepTimer.h"
#include "GeometryType.h"
#include "RenderOption.h"

namespace TessellationVisualization
{
    class TessellationVisualizationRenderer
    {
    public:
        TessellationVisualizationRenderer(const std::shared_ptr<DX::DeviceResources>& deviceResources);
        void CreateDeviceDependentResources();
        void ReleaseDeviceDependentResources();
        void CreateWindowSizeDependentResources();
        void Update(DX::StepTimer const& timer);
        bool Render();

        // Program options
        void SetGeometryType(GeometryType geometryType);
        void SetRenderOption(RenderOption renderOption);

    private:
        // Cached pointer to device resources.
        std::shared_ptr<DX::DeviceResources> m_deviceResources;

        // Resources for rendering geometries
        void GenerateGeometries();
        bool                                                    m_needsRender;
        Microsoft::WRL::ComPtr<ID2D1DrawingStateBlock>          m_stateBlock;
        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>            m_blackBrush;
        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>            m_fillBrush;
        Microsoft::WRL::ComPtr<ID2D1StrokeStyle>                m_strokeStyle;
        D2D1_RECT_F                                             m_geometryBounds;
        GeometryType                                            m_geometryType;
        RenderOption                                            m_renderOption;
        std::vector<Microsoft::WRL::ComPtr<ID2D1PathGeometry>>  m_pathGeometries;
    };
}