﻿#pragma once

#include "Common\StepTimer.h"
#include "Common\DeviceResources.h"
#include "Content\TessellationVisualizationRenderer.h"
#include "Content\GeometryType.h"
#include "Content\RenderOption.h"

// Renders Direct2D and 3D content on the screen.
namespace TessellationVisualization
{
    class TessellationVisualizationMain : public DX::IDeviceNotify
    {
    public:
        TessellationVisualizationMain(const std::shared_ptr<DX::DeviceResources>& deviceResources);
        ~TessellationVisualizationMain();
        void CreateWindowSizeDependentResources();
        void StartTracking() {  }
        void TrackingUpdate(float positionX) {  }
        void StopTracking() {  }
        bool IsTracking() { return false; }
        void StartRenderLoop();
        void StopRenderLoop();
        Concurrency::critical_section& GetCriticalSection() { return m_criticalSection; }

        // IDeviceNotify
        virtual void OnDeviceLost();
        virtual void OnDeviceRestored();

        // Program options
        void SetGeometryType(GeometryType geometryType);
        void SetRenderOption(RenderOption renderOption);

    private:
        void ProcessInput();
        void Update();
        bool Render();

        // Cached pointer to device resources.
        std::shared_ptr<DX::DeviceResources> m_deviceResources;

        // Content renderers.
        std::unique_ptr<TessellationVisualizationRenderer> m_tessellationVisualizationRenderer;

        Windows::Foundation::IAsyncAction^ m_renderLoopWorker;
        Concurrency::critical_section m_criticalSection;

        // Rendering loop timer.
        DX::StepTimer m_timer;

        // Track current input pointer position.
        float m_pointerLocationX;
    };
}