﻿#pragma once

#include <string>
#include "..\Common\DeviceResources.h"
#include "..\Common\StepTimer.h"

namespace TextMorphing
{
    class TextMorphingRenderer
    {
    public:
        TextMorphingRenderer(const std::shared_ptr<DX::DeviceResources>& deviceResources);
        void CreateDeviceDependentResources();
        void ReleaseDeviceDependentResources();
        void CreateWindowSizeDependentResources();
        void Update(DX::StepTimer const& timer);
        void Render();

    private:
        // Cached pointer to device resources.
        std::shared_ptr<DX::DeviceResources> m_deviceResources;

        // Resources for triangle rendering
        struct TriangleInfo
        {
            D2D1_TRIANGLE triangle[2];
        };

        std::vector<TriangleInfo>                       m_triangleInfos;
        D2D1_RECT_F                                     m_geometryBounds;
        Microsoft::WRL::ComPtr<ID2D1Mesh>               m_textMesh;

        // Resources for FPS text rendering.
        std::wstring                                    m_text;
        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_blueBrush;
        Microsoft::WRL::ComPtr<ID2D1DrawingStateBlock>  m_stateBlock;
        Microsoft::WRL::ComPtr<IDWriteTextFormat>		m_textFormat;
    };
}